@@ -18,7 +18,7 @@
  * Copyright (C) 2001 Ramalingam Saravanan. All Rights Reserved.
  *
  * Contributor(s):
- *   Patrick Brunschwig <patrick@mozilla-enigmail.org>
+ *   Patrick Brunschwig <patrick@enigmail.net>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -46,6 +46,7 @@ catch (ex) {
 
 Components.utils.import("resource://enigmail/enigmailCommon.jsm");
 Components.utils.import("resource://enigmail/commonFuncs.jsm");
+Components.utils.import("resource://enigmail/mimeVerify.jsm");
 
 if (! Enigmail) var Enigmail = {};
 
@@ -118,19 +119,18 @@ Enigmail.msg = {
 
     Enigmail.msg.decryptButton = document.getElementById("button-enigmail-decrypt");
 
-    var toolbarElem = document.getElementById("mail-bar2");
-    if (toolbarElem && EnigmailCommon.getOS() == "Darwin") {
-      toolbarElem.setAttribute("platform", "macos");
-    }
-
     // Need to add event listener to Enigmail.msg.messagePane to make it work
     // Adding to msgFrame doesn't seem to work
-    Enigmail.msg.messagePane.addEventListener("unload", Enigmail.msg.messageFrameUnload, true);
+    Enigmail.msg.messagePane.addEventListener("unload", Enigmail.msg.messageFrameUnload.bind(Enigmail.msg), true);
 
-    if (EnigmailCommon.getPref("handleDoubleClick")) {
-      // ovveride function for double clicking an attachment
-      overrideAttribute(["attachmentList"], "onclick",
-                        "Enigmail.msg.enigAttachmentListClick('", "', event);");
+    // override double clicking attachments, but fall back to existing handler if present
+    var attListElem = document.getElementById("attachmentList");
+    if (attListElem) {
+      var newHandler = "Enigmail.msg.enigAttachmentListClick('attachmentList', event)";
+      var oldHandler = attListElem.getAttribute("onclick");
+      if (oldHandler)
+        newHandler = "if (!" + newHandler + ") {" + oldHandler + "}";
+      attListElem.setAttribute("onclick", newHandler);
     }
 
     var treeController = {
@@ -165,6 +165,12 @@ Enigmail.msg = {
     };
 
     top.controllers.appendController(treeController);
+
+    EnigmailCommon.initPrefService();
+    if (EnigmailCommon.getPref("configuredVersion") == "") {
+      EnigmailCommon.setPref("configuredVersion", EnigmailCommon.getVersion());
+      EnigmailFuncs.openSetupWizard(window);
+    }
   },
 
   viewSecurityInfo: function (event, displaySmimeMsg)
@@ -175,11 +181,11 @@ Enigmail.msg = {
       return;
 
     if (gSignatureStatus >= 0 || gEncryptionStatus >= 0) {
-      showMessageReadSecurityInfo()
+      showMessageReadSecurityInfo();
     }
     else {
       if (Enigmail.msg.securityInfo)
-        this.viewOpenpgpInfo()
+        this.viewOpenpgpInfo();
       else
         showMessageReadSecurityInfo();
     }
@@ -250,54 +256,6 @@ Enigmail.msg = {
     Enigmail.msg.securityInfo = null;
   },
 
-  enigMimeInit: function()
-  {
-    // "this" is not Enigmail.msg here
-    EnigmailCommon.DEBUG_LOG("enigmailMessengerOverlay.js: *****enigMimeInit\n");
-
-    try {
-      const enigContenthanderCid =
-        Components.ID("{847b3a51-7ab1-11d4-8f02-006008948af5}");
-
-      const enigEncryptedHanderContract = "@mozilla.org/mimecth;1?type=multipart/encrypted";
-
-      var compMgr = Components.manager.QueryInterface(Components.interfaces.nsIComponentRegistrar);
-
-      var enigContentHandlerCID = compMgr.contractIDToCID(enigEncryptedHanderContract);
-
-      var handlePGPMime = (enigContentHandlerCID.toString() ==
-                       enigContenthanderCid);
-
-      EnigmailCommon.DEBUG_LOG("enigmailMessengerOverlay.js: *****enigMimeInit: handlePGPMime="+handlePGPMime+"\n");
-
-    } catch (ex) {}
-
-
-
-    if (Enigmail.msg.removeListener) {
-      Enigmail.msg.messagePane.removeEventListener("load", Enigmail.msg.enigMimeInit, true);
-      Enigmail.msg.removeListener = false;
-    }
-
-    var enigmailSvc = Enigmail.getEnigmailSvc();
-    if (!enigmailSvc)
-       return;
-
-    if (enigmailSvc.mimeInitialized()) {
-      // Reload message ONLY if enigMimeService has been initialized;
-      // enigMimeInit is only called if enigMimeService was not initialized;
-      // this prevents looping.
-      EnigmailCommon.DEBUG_LOG("enigmailMessengerOverlay.js: *****enigMimeInit: RELOADING MESSAGE\n");
-
-      Enigmail.msg.messageReload(false);
-
-    } else {
-      // Error in MIME initialization; forget saved headers (to avoid looping)
-      Enigmail.msg.savedHeaders = null;
-      EnigmailCommon.ERROR_LOG("enigmailMessengerOverlay.js: *****enigMimeInit: Error in MIME initialization\n");
-    }
-  },
-
   messageFrameUnload: function ()
   {
     EnigmailCommon.DEBUG_LOG("enigmailMessengerOverlay.js: messageFrameUnload\n");
@@ -320,7 +278,8 @@ Enigmail.msg = {
     var viewTypeElementIds = ["messagePaneVertical",
                               "messagePaneClassic",
                               "messagePaneWide"];
-    for (var i = 0; i < viewTypeElementIds.length; i++) {
+    var i;
+    for (i = 0; i < viewTypeElementIds.length; i++) {
       var elementId = viewTypeElementIds[i];
       var element = document.getElementById(elementId);
       if (element) {
@@ -333,7 +292,7 @@ Enigmail.msg = {
     }
 
     var toggleMsgPaneElementIds = ["cmd_toggleMessagePane"];
-    for (var i = 0; i < toggleMsgPaneElementIds.length; i++) {
+    for (i = 0; i < toggleMsgPaneElementIds.length; i++) {
       var elementId = toggleMsgPaneElementIds[i];
       var element = document.getElementById(elementId);
       if (element) {
@@ -350,7 +309,7 @@ Enigmail.msg = {
     ChangeMailLayout(viewType);
 
     // This event requires that we re-subscribe to these events!
-    Enigmail.msg.messagePane.addEventListener("unload", Enigmail.msg.messageFrameUnload, true);
+    Enigmail.msg.messagePane.addEventListener("unload", Enigmail.msg.messageFrameUnload.bind(Enigmail.msg), true);
     this.messageReload(false);
   },
 
@@ -358,7 +317,7 @@ Enigmail.msg = {
     Enigmail.hdrView.statusBarHide();
     MsgToggleMessagePane(true);
 
-    var button=document.getElementById("button_enigmail_decrypt")
+    var button=document.getElementById("button_enigmail_decrypt");
     if (gFolderDisplay.messageDisplay.visible) {
       button.removeAttribute("disabled");
     }
@@ -424,6 +383,10 @@ Enigmail.msg = {
     for (var j=0; j<optList.length; j++) {
       var menuElement = document.getElementById("enigmail_"+optList[j]);
       menuElement.setAttribute("checked", EnigmailCommon.getPref(optList[j]) ? "true" : "false");
+
+      menuElement = document.getElementById("enigmail_"+optList[j]+"2");
+      if (menuElement)
+        menuElement.setAttribute("checked", EnigmailCommon.getPref(optList[j]) ? "true" : "false");
     }
 
     optList = ["decryptverify", "importpublickey", "savedecrypted"];
@@ -435,9 +398,54 @@ Enigmail.msg = {
       else {
          menuElement.removeAttribute("disabled");
       }
+
+      menuElement = document.getElementById("enigmail_"+optList[j]+"2");
+      if (menuElement) {
+        if (Enigmail.msg.decryptButton && Enigmail.msg.decryptButton.disabled) {
+           menuElement.setAttribute("disabled", "true");
+        }
+        else {
+           menuElement.removeAttribute("disabled");
+        }
+      }
     }
   },
 
+  displayMainMenu: function(menuPopup) {
+
+    function traverseTree(currentElement, func)
+    {
+      if (currentElement)
+      {
+        func(currentElement);
+        if (currentElement.id)
+          EnigmailCommon.DEBUG_LOG("traverseTree: "+currentElement.id+"\n");
+
+        // Traverse the tree
+        var i=0;
+        var currentElementChild=currentElement.childNodes[i];
+        while (currentElementChild)
+        {
+          // Recursively traverse the tree structure of the child node
+          traverseTree(currentElementChild, func);
+          i++;
+          currentElementChild=currentElement.childNodes[i];
+        }
+      }
+    }
+
+    var p = menuPopup.parentNode;
+    var a = document.getElementById("menu_EnigmailPopup");
+    var c = a.cloneNode(true);
+    p.removeChild(menuPopup);
+
+
+    traverseTree(c, function _updNode(node) {
+       if (node.id && node.id.length > 0) node.id += "2";
+    });
+    p.appendChild(c);
+
+  },
 
   toggleAttribute: function (attrName)
   {
@@ -477,9 +485,20 @@ Enigmail.msg = {
       isAuto: isAuto
     };
 
+    let contentType = "text/plain";
+    if ('content-type' in currentHeaderData) contentType=currentHeaderData['content-type'].headerValue;
+
+
+    // don't parse message if we know it's a PGP/MIME message
+    if (((contentType.search(/^multipart\/signed(;|$)/i) == 0) && (contentType.search(/application\/pgp-signature/i)>0)) ||
+      ((contentType.search(/^multipart\/encrypted(;|$)/i) == 0) && (contentType.search(/application\/pgp-encrypted/i)>0))) {
+      this.messageDecryptCb(event, isAuto, null);
+      return;
+    }
+
     try {
       if (gFolderDisplay.selectedMessageIsNews) throw "dummy"; // workaround for broken NNTP support in Gloda
-      MsgHdrToMimeMessage(gFolderDisplay.selectedMessage , cbObj, Enigmail.msg.msgDecryptMimeCb, true);
+      MsgHdrToMimeMessage(gFolderDisplay.selectedMessage , cbObj, Enigmail.msg.msgDecryptMimeCb, true, {examineEncryptedParts: true, partsOnDemand: false});
     }
     catch (ex) {
       EnigmailCommon.DEBUG_LOG("enigmailMessengerOverlay.js: enigMessageDecrypt: cannot use MsgHdrToMimeMessage\n");
@@ -502,25 +521,7 @@ Enigmail.msg = {
         var isAuto = argList[1];
         var mimeMsg = argList[2];
         Enigmail.msg.messageDecryptCb(event, isAuto, mimeMsg);
-      }, 0, [this.event, this.isAuto, mimeMsg])
-  },
-
-
-  enigMimeInitialize: function ()
-  {
-    EnigmailCommon.DEBUG_LOG("enigmailMessengerOverlay.js: enigMimeInitialize() - loading enigmail:dummy ...\n");
-
-    // Need to add event listener to Enigmail.msg.messagePane to make it work
-    // Adding to msgFrame doesn't seem to work
-    Enigmail.msg.messagePane.addEventListener("load", Enigmail.msg.enigMimeInit, true);
-    Enigmail.msg.removeListener = true;
-
-    Enigmail.msg.noShowReload = true;
-
-    var msgFrame = EnigmailCommon.getFrame(window, "messagepane");
-    messenger.loadURL(msgFrame, "enigmail:dummy");
-
-    return;
+      }, 0, [this.event, this.isAuto, mimeMsg]);
   },
 
   enumerateMimeParts: function (mimePart, resultObj)
@@ -528,8 +529,21 @@ Enigmail.msg = {
     EnigmailCommon.DEBUG_LOG("enumerateMimeParts: "+mimePart.partName+" - "+mimePart.headers["content-type"]+"\n");
 
     try {
+      if (typeof(mimePart.contentType) == "string" &&
+          mimePart.contentType == "multipart/fake-container") {
+        // workaround for wrong content type of signed message
+        let signedPart = mimePart.parts[1];
+        if (typeof(signedPart.headers["content-type"][0]) == "string") {
+          if (signedPart.headers["content-type"][0].search(/application\/pgp-signature/i) >= 0) {
+            resultObj.signed=signedPart.partName.replace(/\.[0-9]+$/, "");
+            EnigmailCommon.DEBUG_LOG("enumerateMimeParts: found signed subpart "+resultObj.signed + "\n");
+          }
+        }
+      }
+
       var ct = mimePart.headers["content-type"][0];
       if (typeof(ct) == "string") {
+        ct = ct.replace(/[\r\n]/g, " ");
         if (ct.search(/multipart\/signed.*application\/pgp-signature/i) >= 0) {
           resultObj.signed=mimePart.partName;
         }
@@ -569,7 +583,7 @@ Enigmail.msg = {
           headers: {'content-type': contentType },
           contentType: contentType,
           parts: null
-        }
+        };
       }
 
       // Copy selected headers
@@ -590,7 +604,7 @@ Enigmail.msg = {
       var embeddedSigned = null;
       var embeddedEncrypted = null;
 
-      if (mimeMsg.parts != null) {
+      if (mimeMsg.parts != null && Enigmail.msg.savedHeaders["content-type"].search(/^multipart\/encrypted(;|$)/i) != 0) {
         // TB >= 8.0
         var resultObj={ encrypted: "", signed: "" };
         this.enumerateMimeParts(mimeMsg, resultObj);
@@ -603,31 +617,8 @@ Enigmail.msg = {
         if (resultObj.encrypted || resultObj.signed) {
           let mailUrl = this.getCurrentMsgUrl();
           if (mailUrl) {
-              if (resultObj.signed) embeddedSigned = mailUrl.spec+"&part="+resultObj.signed.replace(/\.\d+$/, "");
-              if (resultObj.encrypted) embeddedEncrypted = mailUrl.spec+"&part="+resultObj.encrypted.replace(/\.\d+$/, "");
-          }
-        }
-      }
-
-      if (! (embeddedSigned || embeddedEncrypted)) {
-        // TB <= 7.x
-        if (Enigmail.msg.savedHeaders["content-type"] &&
-            ((Enigmail.msg.savedHeaders["content-type"].search(/^multipart\/mixed/i) == 0) ||
-             (Enigmail.msg.savedHeaders["content-type"].search(/^multipart\/encrypted/i) == 0))) {
-          for (var indexb in currentAttachments) {
-            var attachment = currentAttachments[indexb];
-
-            if (attachment) {
-              if (attachment.contentType.search(/^application\/pgp-signature/i) == 0) {
-                if (! attachment.isExternalAttachment)
-                  embeddedSigned = attachment.url.replace(/\&filename=.*$/,"").replace(/\.\d+\.\d+$/, "");
-              }
-              if (attachment.contentType.search(/^application\/pgp-encrypted/i) == 0) {
-                if (! attachment.isExternalAttachment)
-                  embeddedEncrypted = attachment.url.replace(/\&filename=.*$/,"").replace(/\.\d+\.\d+$/, "");
-              }
-              EnigmailCommon.DEBUG_LOG("enigmailMessengerOverlay.js: mimePart "+indexb+": "+attachment.contentType+"\n");
-            }
+            if (resultObj.signed) embeddedSigned = mailUrl.spec+"?part="+resultObj.signed.replace(/\.\d+$/, "");
+            if (resultObj.encrypted) embeddedEncrypted = mailUrl.spec+"?part="+resultObj.encrypted.replace(/\.\d+$/, "");
           }
         }
       }
@@ -651,7 +642,7 @@ Enigmail.msg = {
           if (!enigmailSvc)
             return;
 
-          if ((!enigmailSvc.mimeInitialized() && encrypedMsg) || signedMsg ||
+          if (signedMsg ||
               ((!encrypedMsg) && (embeddedSigned || embeddedEncrypted))) {
             Enigmail.hdrView.updateHdrIcons(EnigmailCommon.POSSIBLE_PGPMIME, 0, "", "", "", EnigmailCommon.getString("possiblyPgpMime"));
           }
@@ -665,12 +656,6 @@ Enigmail.msg = {
         enigmailSvc = Enigmail.getEnigmailSvc();
         if (!enigmailSvc)
           return;
-
-        if (!enigmailSvc.mimeInitialized()) {
-          // Display enigmail:dummy URL in message pane to initialize
-          this.enigMimeInitialize();
-          return;
-        }
       }
 
       if (((contentType.search(/^multipart\/encrypted(;|$)/i) == 0) ||
@@ -681,11 +666,7 @@ Enigmail.msg = {
         if (!enigmailSvc)
           return;
 
-        if (!enigmailSvc.mimeInitialized()) {
-          this.enigMimeInitialize();
-          return;
-        }
-        else if (! isAuto) {
+        if (! isAuto) {
           Enigmail.msg.messageReload(false);
         }
         else if (embeddedEncrypted && (! encrypedMsg)) {
@@ -729,11 +710,8 @@ Enigmail.msg = {
             Enigmail.msg.verifyEmbeddedMsg(window, mailNewsUrl, msgWindow, msgUriSpec, contentEncoding, event);
           }
           else {
-
-            var verifier = Components.classes[EnigmailCommon.ENIGMIMEVERIFY_CONTRACTID].createInstance(Components.interfaces.nsIEnigMimeVerify);
-
-            verifier.init(window, mailNewsUrl, msgWindow, msgUriSpec,
-                          true, enableSubpartTreatment);
+            var verifier = EnigmailVerify.newVerifier(false, mailNewsUrl, false);
+            verifier.startStreaming(window, msgWindow, msgUriSpec);
 
           }
           return;
@@ -763,7 +741,7 @@ Enigmail.msg = {
 
 
     if (bodyElement.firstChild) {
-      let node = bodyElement.firstChild
+      let node = bodyElement.firstChild;
       while (node) {
         if (node.nodeName == "DIV") {
           foundIndex = node.textContent.indexOf(findStr);
@@ -881,7 +859,7 @@ Enigmail.msg = {
           if (endOffset > 0) {
             subText = subText.substr(0,endOffset) + "\n";
 
-            var matches = subText.match(/\nCharset: *(.*) *\n/i)
+            var matches = subText.match(/\nCharset: *(.*) *\n/i);
             if (matches && (matches.length > 1)) {
               // Override character set
               charset = matches[1];
@@ -972,7 +950,7 @@ Enigmail.msg = {
         msgText = EnigmailCommon.convertToUnicode(msgText, "UTF-8");
         Enigmail.msg.messageParseCallback(msgText, contentEncoding, charset, interactive,
                                           importOnly, messageUrl, null, retry + 1,
-                                          head, tail, msgUriSpec)
+                                          head, tail, msgUriSpec);
         return;
       }
     }
@@ -1071,7 +1049,7 @@ Enigmail.msg = {
 
     Enigmail.msg.noShowReload = true;
 
-    var bodyElement = msgFrame.document.getElementsByTagName("body")[0];
+    bodyElement = msgFrame.document.getElementsByTagName("body")[0];
     if (bodyElement.firstChild) {
       var node = bodyElement.firstChild;
       var foundIndex = -1;
@@ -1183,16 +1161,16 @@ Enigmail.msg = {
   {
     // Escape special characters
     if (text.indexOf("&") > -1)
-      text = text.replace(/&/g, "&amp;")
+      text = text.replace(/&/g, "&amp;");
 
     if (text.indexOf("<") > -1)
-      text = text.replace(/</g, "&lt;")
+      text = text.replace(/</g, "&lt;");
 
     if (text.indexOf(">") > -1)
-      text = text.replace(/>/g, "&gt;")
+      text = text.replace(/>/g, "&gt;");
 
     if (text.indexOf("\"") > -1)
-      text = text.replace(/"/g, "&quot;")
+      text = text.replace(/"/g, "&quot;");
 
     if (!hyperlink)
       return text;
@@ -1383,7 +1361,7 @@ Enigmail.msg = {
         contentData += "\r\n"+Enigmail.msg.decryptedMessage.plainText;
       }
 
-      if (!(enigmailSvc.isDosLike)) {
+      if (!(EnigmailCommon.isDosLike())) {
         contentData = contentData.replace(/\r\n/g, "\n");
       }
     }
@@ -1414,7 +1392,7 @@ Enigmail.msg = {
 
     if (!mailNewsUrl) {
       this.msgDefaultPrint(elementId);
-      return
+      return;
     }
 
     if (Enigmail.msg.decryptedMessage.url != mailNewsUrl.spec) {
@@ -1507,16 +1485,6 @@ Enigmail.msg = {
     if (!mailNewsUrl)
       return;
 
-    var ipcBuffer = Components.classes[EnigmailCommon.IPCBUFFER_CONTRACTID].createInstance(Components.interfaces.nsIIPCBuffer);
-    var mimeListener = Components.classes[EnigmailCommon.ENIGMIMELISTENER_CONTRACTID].createInstance(Components.interfaces.nsIEnigMimeListener);
-
-    if (bufferSize > 0) {
-      ipcBuffer.open(bufferSize, false);
-    }
-    else {
-      ipcBuffer.open(EnigmailCommon.MSG_BUFFER_SIZE, false);
-    }
-
     var callbackArg = { interactive:interactive,
                         importOnly:importOnly,
                         contentEncoding:contentEncoding,
@@ -1524,49 +1492,69 @@ Enigmail.msg = {
                         messageUrl:mailNewsUrl.spec,
                         msgUriSpec:msgUriSpec,
                         signature:signature,
-                        ipcBuffer:ipcBuffer,
-                        expectedBufferSize: bufferSize,
+                        data: "",
                         head:head,
                         tail:tail,
-                        mimeListener: mimeListener,
                         callbackFunction: callbackFunction };
 
-    var requestObserver = EnigmailCommon.newRequestObserver(Enigmail.msg.msgDirectCallback,
-                                                            callbackArg);
+    var msgSvc = messenger.messageServiceFromURI(msgUriSpec);
 
-    ipcBuffer.observe(requestObserver, mailNewsUrl);
+    var listener = {
+      QueryInterface: XPCOMUtils.generateQI([Components.interfaces.nsIStreamListener]),
+      onStartRequest: function() {
+        this.data = "";
+        this.inStream = Components.classes["@mozilla.org/scriptableinputstream;1"].
+          createInstance(Components.interfaces.nsIScriptableInputStream);
 
-    var ioServ = Components.classes[EnigmailCommon.IOSERVICE_CONTRACTID].getService(Components.interfaces.nsIIOService);
+      },
+      onDataAvailable: function(req, sup, stream, offset, count) {
+        this.inStream.init(stream);
+        this.data += this.inStream.read(count);
+      },
+      onStopRequest: function() {
+        var start = this.data.indexOf("-----BEGIN PGP");
+        var end = this.data.indexOf("-----END PGP");
 
-    var channel = ioServ.newChannelFromURI(mailNewsUrl);
+        if (start >= 0 && end > start) {
+          var tStr = this.data.substr(end);
+          var n = tStr.indexOf("\n");
+          var r = tStr.indexOf("\r");
+          var lEnd = -1;
+          if (n >= 0 && r >= 0) {
+            lEnd = Math.min(r, n);
+          }
+          else if (r >= 0) {
+            lEnd = r;
+          }
+          else if (n >= 0)
+            lEnd = n;
 
-    var pipeFilter = Components.classes[EnigmailCommon.PIPEFILTERLISTENER_CONTRACTID].createInstance(Components.interfaces.nsIPipeFilterListener);
-    pipeFilter.init(ipcBuffer, null,
-                  "-----BEGIN PGP",
-                  "-----END PGP",
-                  0, true, false, null);
+          if (lEnd >= 0) {
+            end += lEnd;
+          }
 
-    var listener;
+          callbackArg.data = this.data.substring(start, end+1);
+          EnigmailCommon.DEBUG_LOG("enigmailMessengerOverlay.js: data: >"+callbackArg.data+"<\n");
+          Enigmail.msg.msgDirectCallback(callbackArg);
+        }
+      }
+    };
 
-    try {
+    msgSvc.streamMessage(msgUriSpec,
+                    listener,
+                    msgWindow,
+                    null,
+                    false,
+                    null,
+                    false);
 
-      mimeListener.init(pipeFilter, null, EnigmailCommon.MSG_HEADER_SIZE, true, false, true);
-
-      listener = mimeListener;
-
-    } catch (ex) {
-      listener = pipeFilter;
-    }
-
-    channel.asyncOpen(pipeFilter, mailNewsUrl);
   },
 
 
-  msgDirectCallback: function (cbArray)
+  msgDirectCallback: function (callbackArg)
   {
     EnigmailCommon.DEBUG_LOG("enigmailMessengerOverlay.js: msgDirectCallback: \n");
 
-    var callbackArg = cbArray[0];
     var mailNewsUrl = Enigmail.msg.getCurrentMsgUrl();
     var urlSpec = mailNewsUrl ? mailNewsUrl.spec : "";
     var newBufferSize = 0;
@@ -1578,33 +1566,9 @@ Enigmail.msg = {
       return;
     }
 
-    if (callbackArg.ipcBuffer.overflowed) {
-      WARNING_LOG("enigmailMessengerOverlay.js: msgDirectCallback: MESSAGE BUFFER OVERFLOW\n");
-      if (! callbackArg.expectedBufferSize) {
-        // set correct buffer size
-        newBufferSize=((callbackArg.ipcBuffer.totalBytes+1500)/1024).toFixed(0)*1024;
-      }
-    }
-
-    var msgText = callbackArg.ipcBuffer.getData();
+    var msgText = callbackArg.data;
     msgText = EnigmailCommon.convertFromUnicode(msgText, "UTF-8");
 
-    callbackArg.ipcBuffer.shutdown();
-
-    if (newBufferSize > 0) {
-      // retry with correct buffer size
-      Enigmail.msg.msgDirectDecrypt(callbackArg.interactive,
-                                    callbackArg.importOnly,
-                                    callbackArg.contentEncoding,
-                                    callbackArg.charset,
-                                    callbackArg.signature,
-                                    newBufferSize,
-                                    callbackArg.head,
-                                    callbackArg.tail,
-                                    callbackArg.msgUriSpec,
-                                    callbackArg.callbackFunction);
-
-    }
     EnigmailCommon.DEBUG_LOG("enigmailMessengerOverlay.js: msgDirectCallback: msgText='"+msgText+"'\n");
 
     var f = function (argList) {
@@ -1620,7 +1584,7 @@ Enigmail.msg = {
                            cb.head,
                            cb.tail,
                            cb.msgUriSpec);
-    }
+    };
 
     EnigmailCommon.dispatchEvent(f, 0, [msgText, callbackArg ]);
   },
@@ -1630,10 +1594,7 @@ Enigmail.msg = {
   {
     EnigmailCommon.DEBUG_LOG("enigmailMessengerOverlay.js: verifyEmbeddedMsg: msgUrl"+msgUrl+"\n");
 
-    var ipcBuffer = Components.classes[EnigmailCommon.IPCBUFFER_CONTRACTID].createInstance(Components.interfaces.nsIIPCBuffer);
-    ipcBuffer.open(-1, false);
-
-    var callbackArg = { ipcBuffer: ipcBuffer,
+    var callbackArg = { data: "",
                         window: window,
                         msgUrl: msgUrl,
                         msgWindow: msgWindow,
@@ -1641,42 +1602,36 @@ Enigmail.msg = {
                         contentEncoding: contentEncoding,
                         event: event };
 
-    var requestObserver = EnigmailCommon.newRequestObserver(Enigmail.msg.verifyEmbeddedCallback,
-                                                            callbackArg);
+    var requestCallback = function _cb (data) {
+      callbackArg.data = data;
+      Enigmail.msg.verifyEmbeddedCallback(callbackArg);
+    };
 
-    ipcBuffer.observe(requestObserver, msgUrl);
+    var bufferListener = EnigmailCommon.newStringStreamListener(requestCallback);
 
     var ioServ = Components.classes[EnigmailCommon.IOSERVICE_CONTRACTID].getService(Components.interfaces.nsIIOService);
 
     var channel = ioServ.newChannelFromURI(msgUrl);
 
-    channel.asyncOpen(ipcBuffer, msgUrl);
+    channel.asyncOpen(bufferListener, msgUrl);
   },
 
-  verifyEmbeddedCallback: function (cbArray)
+  verifyEmbeddedCallback: function (callbackArg)
   {
     EnigmailCommon.DEBUG_LOG("enigmailMessengerOverlay.js: verifyEmbeddedCallback: \n");
 
-    var callbackArg = cbArray[0];
-    var txt = callbackArg.ipcBuffer.getData();
-    callbackArg.ipcBuffer.shutdown();
-
-    if (txt.length > 0) {
-      msigned=txt.search(/content\-type:[ \t]*multipart\/signed/i);
+    if (callbackArg.data.length > 0) {
+      let msigned=callbackArg.data.search(/content\-type:[ \t]*multipart\/signed/i);
       if(msigned >= 0) {
+
         // Real multipart/signed message; let's try to verify it
         EnigmailCommon.DEBUG_LOG("enigmailMessengerOverlay.js: verifyEmbeddedCallback: detected multipart/signed. msigned: "+msigned+"\n");
 
-        callbackArg.enableSubpartTreatment=(msigned > 0);
+        let enableSubpartTreatment=(msigned > 0);
 
-        var uri = Components.classes[EnigmailCommon.SIMPLEURI_CONTRACTID].createInstance(Components.interfaces.nsIURI);
-        uri.spec = "enigmail:dummy";
+        var verifier = EnigmailVerify.newVerifier(enableSubpartTreatment, callbackArg.mailNewsUrl, true);
+        verifier.verifyData(callbackArg.window, callbackArg.msgWindow, callbackArg.msgUriSpec, callbackArg.data);
 
-        var channel = EnigmailCommon.newStringChannel(uri, "", "", txt);
-        var verifier = Components.classes[EnigmailCommon.ENIGMIMEVERIFY_CONTRACTID].createInstance(Components.interfaces.nsIEnigMimeVerify);
-
-        verifier.initWithChannel(callbackArg.window, channel, callbackArg.msgWindow, callbackArg.msgUriSpec,
-                        true, callbackArg.enableSubpartTreatment);
         return;
       }
     }
@@ -1715,15 +1670,15 @@ Enigmail.msg = {
     var anAttachment = selectedAttachments[0];
 
     switch (actionType) {
-    case "saveAttachment":
-    case "openAttachment":
-    case "importKey":
-    case "revealName":
-      this.handleAttachment(actionType, anAttachment);
-      break;
-    case "verifySig":
-      this.verifyDetachedSignature(anAttachment);
-      break;
+      case "saveAttachment":
+      case "openAttachment":
+      case "importKey":
+      case "revealName":
+        this.handleAttachment(actionType, anAttachment);
+        break;
+      case "verifySig":
+        this.verifyDetachedSignature(anAttachment);
+        break;
     }
   },
 
@@ -1780,7 +1735,7 @@ Enigmail.msg = {
     var tmpDir = EnigmailCommon.getTempDir();
     var outFile1, outFile2;
     outFile1 = Components.classes[EnigmailCommon.LOCAL_FILE_CONTRACTID].
-      createInstance(Components.interfaces.nsILocalFile);
+      createInstance(EnigmailCommon.getLocalFileApi());
     outFile1.initWithPath(tmpDir);
     if (!(outFile1.isDirectory() && outFile1.isWritable())) {
       EnigmailCommon.alert(window, EnigmailCommon.getString("noTempDir"));
@@ -1791,7 +1746,7 @@ Enigmail.msg = {
     this.writeUrlToFile(origAtt.url, outFile1);
 
     outFile2 = Components.classes[EnigmailCommon.LOCAL_FILE_CONTRACTID].
-      createInstance(Components.interfaces.nsILocalFile);
+      createInstance(EnigmailCommon.getLocalFileApi());
     outFile2.initWithPath(tmpDir);
     outFile2.append(this.getAttachmentName(signatureAtt));
     outFile2.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0600);
@@ -1802,7 +1757,7 @@ Enigmail.msg = {
     var r = enigmailSvc.verifyAttachment(window, outFile1, outFile2, statusFlagsObj, errorMsgObj);
 
     if (r == 0)
-      EnigmailCommon.alert(window, EnigmailCommon.getString("signature.verifiedOK", [ this.getAttachmentName(origAtt) ]));
+      EnigmailCommon.alert(window, EnigmailCommon.getString("signature.verifiedOK", [ this.getAttachmentName(origAtt) ]) +"\n\n"+ errorMsgObj.value);
     else
       EnigmailCommon.alert(window, EnigmailCommon.getString("signature.verifyFailed", [ this.getAttachmentName(origAtt) ])+"\n\n"+
         errorMsgObj.value);
@@ -1840,32 +1795,30 @@ Enigmail.msg = {
       fstream.close();
 
     // Close the input stream
-    istream.close()
+    istream.close();
   },
 
   handleAttachment: function (actionType, anAttachment)
   {
     EnigmailCommon.DEBUG_LOG("enigmailMessengerOverlay.js: handleAttachment: actionType="+actionType+", anAttachment(url)="+anAttachment.url+"\n");
 
-    var ipcBuffer = Components.classes[EnigmailCommon.IPCBUFFER_CONTRACTID].createInstance(Components.interfaces.nsIIPCBuffer);
-
     var argumentsObj = { actionType: actionType,
                          attachment: anAttachment,
                          forceBrowser: false,
-                         ipcBuffer: ipcBuffer
+                         data: ""
                        };
 
-    var requestObserver = EnigmailCommon.newRequestObserver(Enigmail.msg.decryptAttachmentCallback,
-                                                            argumentsObj);
+    var f = function _cb(data) {
+      argumentsObj.data = data;
+      Enigmail.msg.decryptAttachmentCallback([argumentsObj]);
+    };
 
+    var bufferListener = EnigmailCommon.newStringStreamListener(f);
     var ioServ = Components.classes[EnigmailCommon.IOSERVICE_CONTRACTID].getService(Components.interfaces.nsIIOService);
-
-    ipcBuffer.open(-1, false);
     var msgUri = ioServ.newURI(argumentsObj.attachment.url, null, null);
 
-    ipcBuffer.observe(requestObserver, msgUri);
     var channel = ioServ.newChannelFromURI(msgUri);
-    channel.asyncOpen(ipcBuffer, msgUri);
+    channel.asyncOpen(bufferListener, msgUri);
   },
 
   setAttachmentName: function (attachment, newLabel, index)
@@ -1904,10 +1857,6 @@ Enigmail.msg = {
     var callbackArg = cbArray[0];
     const nsIEnigmail = Components.interfaces.nsIEnigmail;
 
-    if (callbackArg.ipcBuffer.overflowed) {
-      WARNING_LOG("enigmailMessengerOverlay.js: decryptAttachmentCallback: MESSAGE BUFFER OVERFLOW\n");
-    }
-
     var exitCodeObj = new Object();
     var statusFlagsObj = new Object();
     var errorMsgObj= new Object();
@@ -1919,7 +1868,7 @@ Enigmail.msg = {
     var rawFileName=Enigmail.msg.getAttachmentName(callbackArg.attachment).replace(/\.(asc|pgp|gpg)$/i,"");
 
     if (callbackArg.actionType != "importKey") {
-      origFilename = enigmailSvc.getAttachmentFileName(window, callbackArg.ipcBuffer);
+      origFilename = EnigmailCommon.getAttachmentFileName(window, callbackArg.data);
       if (origFilename && origFilename.length > rawFileName.length) rawFileName = origFilename;
     }
 
@@ -1940,7 +1889,7 @@ Enigmail.msg = {
       // open
       var tmpDir = EnigmailCommon.getTempDir();
       try {
-        outFile = Components.classes[EnigmailCommon.LOCAL_FILE_CONTRACTID].createInstance(Components.interfaces.nsILocalFile);
+        outFile = Components.classes[EnigmailCommon.LOCAL_FILE_CONTRACTID].createInstance(EnigmailCommon.getLocalFileApi());
         outFile.initWithPath(tmpDir);
         if (!(outFile.isDirectory() && outFile.isWritable())) {
           errorMsgObj.value=EnigmailCommon.getString("noTempDir");
@@ -1957,9 +1906,7 @@ Enigmail.msg = {
 
     if (callbackArg.actionType == "importKey") {
       try {
-        var dataLength = new Object();
-        var byteData = callbackArg.ipcBuffer.getByteData(dataLength);
-        exitStatus = enigmailSvc.importKey(parent, 0, byteData, "", errorMsgObj);
+        exitStatus = enigmailSvc.importKey(parent, 0, callbackArg.data, "", errorMsgObj);
       }
       catch (ex) {}
       if (exitStatus == 0) {
@@ -1974,15 +1921,15 @@ Enigmail.msg = {
 
     exitStatus=enigmailSvc.decryptAttachment(window, outFile,
                                   Enigmail.msg.getAttachmentName(callbackArg.attachment),
-                                  callbackArg.ipcBuffer,
+                                  callbackArg.data,
                                   exitCodeObj, statusFlagsObj,
                                   errorMsgObj);
 
-    callbackArg.ipcBuffer.shutdown();
     if ((! exitStatus) || exitCodeObj.value != 0) {
       exitStatus=false;
-      if (statusFlagsObj.value &
-          (nsIEnigmail.DECRYPTION_OKAY | nsIEnigmail.UNVERIFIED_SIGNATURE)) {
+      if ((statusFlagsObj.value & nsIEnigmail.DECRYPTION_OKAY) &&
+         (statusFlagsObj.value & nsIEnigmail.UNVERIFIED_SIGNATURE)) {
+
         if (callbackArg.actionType == "openAttachment") {
           exitStatus = EnigmailCommon.confirmDlg(window, EnigmailCommon.getString("decryptOkNoSig"), EnigmailCommon.getString("msgOvl.button.contAnyway"));
         }
@@ -2006,7 +1953,7 @@ Enigmail.msg = {
                (callbackArg.actionType == "openAttachment")) {
         var ioServ = Components.classes[EnigmailCommon.IOSERVICE_CONTRACTID].getService(Components.interfaces.nsIIOService);
         var outFileUri = ioServ.newFileURI(outFile);
-        var fileExt = outFile.leafName.replace(/(.*\.)(\w+)$/, "$2")
+        var fileExt = outFile.leafName.replace(/(.*\.)(\w+)$/, "$2");
         if (fileExt && ! callbackArg.forceBrowser) {
           var extAppLauncher = Components.classes[EnigmailCommon.MIME_CONTRACTID].getService(Components.interfaces.nsPIExternalAppLauncher);
           extAppLauncher.deleteTemporaryFileOnExit(outFile);
@@ -2087,13 +2034,13 @@ Enigmail.msg = {
 
     var attachment=event.target.attachment;
     if (this.checkEncryptedAttach(attachment)) {
-      if (event.button != 0) return;
-
-      if (event.detail == 2) { // double click
+      if (event.button == 0 && event.detail == 2) { // double click
         this.handleAttachment("openAttachment", attachment);
         event.stopPropagation();
+        return true;
       }
     }
+    return false;
   },
 
   // download keys
@@ -2126,7 +2073,7 @@ Enigmail.msg = {
 
     try {
       var localFile = Components.classes[EnigmailCommon.LOCAL_FILE_CONTRACTID].
-        createInstance(Components.interfaces.nsILocalFile);
+        createInstance(EnigmailCommon.getLocalFileApi());
 
       localFile.initWithPath(filePath);
 
@@ -2180,5 +2127,5 @@ Enigmail.msg = {
   }
 };
 
-window.addEventListener("load",   Enigmail.msg.messengerStartup, false);
+window.addEventListener("load",   Enigmail.msg.messengerStartup.bind(Enigmail.msg), false);
 
