@@ -18,7 +18,7 @@
  * Copyright (C) 2001 Ramalingam Saravanan. All Rights Reserved.
  *
  * Contributor(s):
- *   Patrick Brunschwig <patrick@mozilla-enigmail.org>
+ *   Patrick Brunschwig <patrick@enigmail.net>
  *   Ludwig Hügelschäfer <ludwig@hammernoch.net>
  *
  * Alternatively, the contents of this file may be used under the terms of
@@ -47,11 +47,16 @@ catch (ex) {
 Components.utils.import("resource://enigmail/enigmailCommon.jsm");
 Components.utils.import("resource://enigmail/commonFuncs.jsm");
 
+try {
+  Components.utils.import("resource:///modules/MailUtils.js");
+}
+catch(ex) {}
+
+
 if (! Enigmail) var Enigmail = {};
 
 Enigmail.msg = {
 // List of hash algorithms for PGP/MIME signatures
-  mimeHashAlgo: [null, "sha1", "ripemd160", "sha256", "sha384", "sha512", "sha224"],
   editor: null,
   dirty: null,
   processed: null,
@@ -59,6 +64,7 @@ Enigmail.msg = {
   sendPgpMime: false,
   sendMode: null,
   sendModeDirty: 0,
+  sendProcess: false,
   nextCommandId: null,
   docaStateListener: null,
   identity: null,
@@ -159,7 +165,7 @@ Enigmail.msg = {
         r=this.identity.getBoolAttribute(value);
         break;
       }
-      EnigmailCommon.DEBUG_LOG("  "+value+"="+r+"\n");
+      EnigmailCommon.DEBUG_LOG("enigmailMsgComposeOverlay.js: Enigmail.msg.getAccDefault: "+value+"="+r+"\n");
       return r;
     }
     else {
@@ -255,7 +261,6 @@ Enigmail.msg = {
     EnigmailCommon.DEBUG_LOG("enigmailMsgComposeOverlay.js: Enigmail.msg.getMsgPropertiesCb\n");
 
     const nsIEnigmail = Components.interfaces.nsIEnigmail;
-    const SIGN    = nsIEnigmail.SEND_SIGNED;
 
     var flags = 0;
     if (mimeMsg && mimeMsg.headers["x-enigmail-draft-status"])
@@ -263,7 +268,9 @@ Enigmail.msg = {
 
     EnigmailCommon.DEBUG_LOG("enigmailMsgComposeOverlay.js: Enigmail.msg.getMsgPropertiesCb: draftStatus: "+flags+"\n");
 
-    if (flags & SIGN) Enigmail.msg.setSendMode('sign');
+    if (flags & nsIEnigmail.SEND_SIGNED) Enigmail.msg.setSendMode('sign');
+    if (flags & nsIEnigmail.SEND_ENCRYPTED) Enigmail.msg.setSendMode('encrypt');
+
   },
 
 
@@ -419,6 +426,7 @@ Enigmail.msg = {
     this.sendMode = 0;
     this.enableRules = true;
     this.identity = null;
+    this.sendProcess = false;
 
     if (! closing) {
       this.setIdentityDefaults();
@@ -683,6 +691,29 @@ Enigmail.msg = {
     this.editorInsertText(text);
   },
 
+
+  getMsgFolderFromUri:  function(uri, checkFolderAttributes)
+  {
+    let msgfolder = null;
+    if (typeof MailUtils != 'undefined') {
+      return MailUtils.getFolderForURI(uri, checkFolderAttributes);
+    }
+    try {
+      // Postbox, older versions of TB
+      let resource = GetResourceFromUri(uri);
+      msgfolder = resource.QueryInterface(Components.interfaces.nsIMsgFolder);
+      if (checkFolderAttributes) {
+        if (!(msgfolder && (msgfolder.parent || msgfolder.isServer))) {
+          msgfolder = null;
+        }
+      }
+    }
+    catch (ex) {
+       //dump("failed to get the folder resource\n");
+    }
+    return msgfolder;
+  },
+
   goAccountManager: function ()
   {
     EnigmailCommon.DEBUG_LOG("enigmailMsgComposeOverlay.js: Enigmail.msg.goAccountManager:\n");
@@ -691,16 +722,18 @@ Enigmail.msg = {
     try {
         var currentId=getCurrentIdentity();
         var amService=Components.classes["@mozilla.org/messenger/account-manager;1"].getService();
-        var servers;
+        var servers, folderURI;
         try {
           // Gecko >= 20
           servers=amService.getServersForIdentity(currentId);
+          folderURI=servers.queryElementAt(0, Components.interfaces.nsIMsgIncomingServer).serverURI;
         }
         catch(ex) {
           servers=amService.GetServersForIdentity(currentId);
+          folderURI=servers.GetElementAt(0).QueryInterface(Components.interfaces.nsIMsgIncomingServer).serverURI;
         }
-        var folderURI=servers.GetElementAt(0).QueryInterface(Components.interfaces.nsIMsgIncomingServer).serverURI;
-        server=GetMsgFolderFromUri(folderURI, true).server
+
+        server=this.getMsgFolderFromUri(folderURI, true).server;
     } catch (ex) {}
     window.openDialog("chrome://enigmail/content/am-enigprefs-edit.xul", "", "dialog,modal,centerscreen", {identity: currentId, account: server});
     this.setIdentityDefaults();
@@ -896,29 +929,31 @@ Enigmail.msg = {
     if (this.getAccDefault("enabled")) {
       var compFields = Components.classes["@mozilla.org/messengercompose/composefields;1"].createInstance(Components.interfaces.nsIMsgCompFields);
       Recipients2CompFields(compFields);
+      recipientsSelection = EnigmailCommon.getPref("recipientsSelection");
+
       var arrLen = new Object();
       var matchedKeysObj = new Object();
       var flagsObj = new Object();
       var toAddrList = new Array();
       var recList;
       if (compFields.to.length > 0) {
-        recList = compFields.splitRecipients(compFields.to, true, arrLen)
+        recList = compFields.splitRecipients(compFields.to, true, arrLen);
         this.addRecipients(toAddrList, recList);
       }
 
       if (compFields.cc.length > 0) {
-        recList = compFields.splitRecipients(compFields.cc, true, arrLen)
+        recList = compFields.splitRecipients(compFields.cc, true, arrLen);
         this.addRecipients(toAddrList, recList);
       }
 
       if (compFields.bcc.length > 0) {
-        recList = compFields.splitRecipients(compFields.bcc, true, arrLen)
+        recList = compFields.splitRecipients(compFields.bcc, true, arrLen);
         this.addRecipients(toAddrList, recList);
       }
 
       this.signRules    = 1;
       this.encryptRules = 1;
-      if (toAddrList.length > 0) {
+      if (toAddrList.length > 0 && recipientsSelection != 3 && recipientsSelection != 4) {
         if (Enigmail.hlp.getRecipientsKeys(toAddrList.join(", "), false, false, matchedKeysObj, flagsObj)) {
           this.signRules    = flagsObj.sign;
           this.encryptRules = flagsObj.encrypt;
@@ -987,7 +1022,7 @@ Enigmail.msg = {
     }
   },
 
-  confirmBeforeSend: function (toAddr, gpgKeys, sendFlags, isOffline, msgSendType)
+  confirmBeforeSend: function (toAddr, gpgKeys, sendFlags, isOffline)
   {
     EnigmailCommon.DEBUG_LOG("enigmailMsgComposeOverlay.js: Enigmail.msg.confirmBeforeSend: sendFlags="+sendFlags+"\n");
     // get confirmation before sending message
@@ -1033,6 +1068,7 @@ Enigmail.msg = {
 
   setDraftStatus: function (sendFlags)
   {
+    EnigmailCommon.DEBUG_LOG("enigmailMsgComposeOverlay.js: Enigmail.msg.setDraftStatus - enabling draft mode\n");
     gMsgCompose.compFields.otherRandomHeaders += "X-Enigmail-Draft-Status: "+sendFlags+"\r\n";
   },
 
@@ -1048,8 +1084,9 @@ Enigmail.msg = {
 
         var mesg = EnigmailCommon.getString("composeSpecifyEmail");
 
-        var valueObj = new Object();
-        valueObj.value = userIdValue;
+        var valueObj = {
+          value: userIdValue
+        };
 
         if (EnigmailCommon.promptValue(window, mesg, valueObj)) {
           userIdValue = valueObj.value;
@@ -1080,6 +1117,7 @@ Enigmail.msg = {
     const ENCRYPT = nsIEnigmail.SEND_ENCRYPTED;
 
     var recipientsSelection = EnigmailCommon.getPref("recipientsSelection");
+
     var toAddr = toAddrList.join(", ");
     var bccAddr = bccAddrList.join(", ");
     var testCipher = null;
@@ -1088,7 +1126,7 @@ Enigmail.msg = {
 
     if (toAddr.length>=1) {
 
-       EnigmailCommon.DEBUG_LOG("enigmailMsgComposeOverlay.js: Enigmail.msg.keySelection: toAddr="+toAddr+"\n");
+       EnigmailCommon.DEBUG_LOG("enigmailMsgComposeOverlay.js: Enigmail.msg.keySelection: recipientsSelection= " + recipientsSelection + " / toAddr="+toAddr+"\n");
        var repeatSelection=0;
        while (repeatSelection<2) {
          if (recipientsSelection != 3 && recipientsSelection != 4
@@ -1158,7 +1196,7 @@ Enigmail.msg = {
                                optSendFlags ;
 
            // test recipients
-           testCipher = enigmailSvc.encryptMessage(window, testUiFlags, null,
+           testCipher = enigmailSvc.encryptMessage(window, testUiFlags,
                                                    testPlain,
                                                    fromAddr, toAddr, bccAddr,
                                                    testSendFlags,
@@ -1169,7 +1207,7 @@ Enigmail.msg = {
 
            if (testStatusFlagsObj.value) {
              // check if own key is invalid
-             let s = new RegExp("^\\[GNUPG:\\] INV_(RECP|SGNR) [0-9]+ \\<?" + fromAddr + "\\>?", "m");
+             let s = new RegExp("^INV_(RECP|SGNR) [0-9]+ \\<?" + fromAddr + "\\>?", "m");
              if (testErrorMsgObj.value.search(s) >= 0)  {
                EnigmailCommon.alert(window, EnigmailCommon.getString("errorKeyUnusable", [ fromAddr ]));
                return null;
@@ -1187,7 +1225,7 @@ Enigmail.msg = {
                inputObj.invalidAddr = Enigmail.hlp.getInvalidAddress(testErrorMsgObj.value);
                inputObj.options = "multisel";
                if (recipientsSelection==2)
-                 inputObj.options += ",rulesOption"
+                 inputObj.options += ",rulesOption";
                if (notSignedIfNotEnc)
                  inputObj.options += ",notsigned";
                if (recipientsSelection == 4)
@@ -1256,19 +1294,23 @@ Enigmail.msg = {
     const nsIEnigmail = Components.interfaces.nsIEnigmail;
     const SIGN    = nsIEnigmail.SEND_SIGNED;
     const ENCRYPT = nsIEnigmail.SEND_ENCRYPTED;
+    const CiMsgCompDeliverMode = Components.interfaces.nsIMsgCompDeliverMode;
     var promptSvc = EnigmailCommon.getPromptSvc();
 
     var gotSendFlags = this.sendMode;
     var sendFlags=0;
     window.enigmailSendFlags=0;
 
+
     switch (msgSendType) {
-    case nsIMsgCompDeliverMode.Later:
+    case CiMsgCompDeliverMode.Later:
+      EnigmailCommon.DEBUG_LOG("enigmailMsgComposeOverlay.js: Enigmail.msg.encryptMsg: adding SEND_LATER\n")
       sendFlags |= nsIEnigmail.SEND_LATER;
       break;
-    case nsIMsgCompDeliverMode.SaveAsDraft:
-    case nsIMsgCompDeliverMode.SaveAsTemplate:
-    case nsIMsgCompDeliverMode.AutoSaveAsDraft:
+    case CiMsgCompDeliverMode.SaveAsDraft:
+    case CiMsgCompDeliverMode.SaveAsTemplate:
+    case CiMsgCompDeliverMode.AutoSaveAsDraft:
+      EnigmailCommon.DEBUG_LOG("enigmailMsgComposeOverlay.js: Enigmail.msg.encryptMsg: adding SAVE_MESSAGE\n")
       sendFlags |= nsIEnigmail.SAVE_MESSAGE;
       break;
     }
@@ -1294,6 +1336,8 @@ Enigmail.msg = {
     }
     var encryptIfPossible = false;
     if (sendFlags & nsIEnigmail.SAVE_MESSAGE) {
+      this.setDraftStatus(sendFlags);
+
       if (!((sendFlags & ENCRYPT) && EnigmailCommon.confirmPref(window, EnigmailCommon.getString("savingMessage"), "saveEncrypted",
               EnigmailCommon.getString("msgCompose.button.encrypt"),
               EnigmailCommon.getString("msgCompose.button.dontEncrypt")))) {
@@ -1301,7 +1345,6 @@ Enigmail.msg = {
 
         if (this.attachOwnKeyObj.appendAttachment) this.attachOwnKey();
 
-        if (sendFlags & SIGN) this.setDraftStatus(sendFlags);
         return true;
       }
     }
@@ -1403,23 +1446,24 @@ Enigmail.msg = {
        else {
          var splitRecipients;
          var arrLen =  new Object();
+         var recList;
          splitRecipients = msgCompFields.splitRecipients;
 
          //EnigmailCommon.alert(window, typeof(msgCompFields.cc));
          if (msgCompFields.to.length > 0) {
-           var recList = splitRecipients(msgCompFields.to, true, arrLen)
+           recList = splitRecipients(msgCompFields.to, true, arrLen);
            this.addRecipients(toAddrList, recList);
          }
 
          if (msgCompFields.cc.length > 0) {
-           recList = splitRecipients(msgCompFields.cc, true, arrLen)
+           recList = splitRecipients(msgCompFields.cc, true, arrLen);
            this.addRecipients(toAddrList, recList);
          }
 
          if (msgCompFields.bcc.length > 0) {
-           recList = splitRecipients(msgCompFields.bcc, true, arrLen)
+           recList = splitRecipients(msgCompFields.bcc, true, arrLen);
 
-           var bccLC = EnigmailFuncs.stripEmail(msgCompFields.bcc).toLowerCase()
+           var bccLC = EnigmailFuncs.stripEmail(msgCompFields.bcc).toLowerCase();
            EnigmailCommon.DEBUG_LOG("enigmailMsgComposeOverlay.js: Enigmail.msg.encryptMsg: BCC: "+bccLC+"\n");
 
            var selfBCC = this.identity.email && (this.identity.email.toLowerCase() == bccLC);
@@ -1522,13 +1566,13 @@ Enigmail.msg = {
 
        if ( hasAttachments &&
           (sendFlags & (ENCRYPT | SIGN)) &&
-          !(sendFlags & nsIEnigmail.SEND_PGP_MIME) &&
-          enigmailSvc.composeSecure) {
+          !(sendFlags & nsIEnigmail.SEND_PGP_MIME)) {
 
-          inputObj = new Object();
-          inputObj.pgpMimePossible = true;
-          inputObj.inlinePossible = true;
-          inputObj.restrictedScenario = false;
+          inputObj = {
+            pgpMimePossible: true,
+            inlinePossible: true,
+            restrictedScenario: false
+          };
 
           // determine if attachments are all local files (currently the only
           // supported kind of attachments)
@@ -1541,8 +1585,9 @@ Enigmail.msg = {
           }
 
           if (inputObj.pgpMimePossible || inputObj.inlinePossible) {
-            resultObj = new Object();
-            resultObj.selected = EnigmailCommon.getPref("encryptAttachments");
+            resultObj = {
+              selected: EnigmailCommon.getPref("encryptAttachments")
+            };
 
             //skip or not
             var skipCheck=EnigmailCommon.getPref("encryptAttachmentsSkipDlg");
@@ -1583,16 +1628,6 @@ Enigmail.msg = {
        var usingPGPMime = (sendFlags & nsIEnigmail.SEND_PGP_MIME) &&
                           (sendFlags & (ENCRYPT | SIGN));
 
-       if (usingPGPMime && !enigmailSvc.composeSecure) {
-         if (!EnigmailCommon.confirmDlg(window, EnigmailCommon.getString("noPGPMIME"),
-                EnigmailCommon.getString("msgCompose.button.useInlinePGP"))) {
-            throw Components.results.NS_ERROR_FAILURE;
-
-         }
-
-         usingPGPMime = false;
-       }
-
        // Detect PGP/MIME and S/MIME
        if (usingPGPMime) {
           if (gMsgCompose.compFields.securityInfo instanceof Components.interfaces.nsIMsgSMIMECompFields) {
@@ -1602,7 +1637,7 @@ Enigmail.msg = {
                  var prefAlgo = EnigmailCommon.getPref("mimePreferPgp");
                  if (prefAlgo == 1) {
                    var checkedObj={ value: null};
-                   var prefAlgo = promptSvc.confirmEx(window,
+                   prefAlgo = promptSvc.confirmEx(window,
                               EnigmailCommon.getString("enigConfirm"),
                               EnigmailCommon.getString("pgpMime_sMime.dlg.text"),
                               (promptSvc. BUTTON_TITLE_IS_STRING * promptSvc.BUTTON_POS_0) +
@@ -1671,7 +1706,6 @@ Enigmail.msg = {
          newSecurityInfo.senderEmailAddr = fromAddr;
          newSecurityInfo.recipients = toAddr;
          newSecurityInfo.bccRecipients = bccAddr;
-         newSecurityInfo.hashAlgorithm = this.mimeHashAlgo[EnigmailCommon.getPref("mimeHashAlgorithm")];
 
          EnigmailCommon.DEBUG_LOG("enigmailMsgComposeOverlay.js: Enigmail.msg.encryptMsg: securityInfo = "+newSecurityInfo+"\n");
 
@@ -1694,8 +1728,9 @@ Enigmail.msg = {
          }
        }
 
+       var ioService = EnigmailCommon.getIoService();
        // EnigSend: Handle both plain and encrypted messages below
-       var isOffline = (gIOService && gIOService.offline);
+       var isOffline = (ioService && ioService.offline);
        window.enigmailSendFlags=sendFlags;
 
        // update the list of attachments
@@ -1754,7 +1789,7 @@ Enigmail.msg = {
   {
     // sign/encrpyt message using inline-PGP
 
-    const dce = Components.interfaces.nsIDocumentEncoder
+    const dce = Components.interfaces.nsIDocumentEncoder;
     const nsIEnigmail = Components.interfaces.nsIEnigmail;
     const SIGN    = nsIEnigmail.SEND_SIGNED;
     const ENCRYPT = nsIEnigmail.SEND_ENCRYPTED;
@@ -1811,7 +1846,7 @@ Enigmail.msg = {
 
           if (wrapWidth > 0 && wrapWidth < 68 && gMsgCompose.wrapLength > 0) {
             if (EnigmailCommon.confirmDlg(window, EnigmailCommon.getString("minimalLineWrapping", [ wrapWidth ] ))) {
-              EnigmailCommon.prefRoot.setIntPref("editor.htmlWrapColumn", 68)
+              EnigmailCommon.prefRoot.setIntPref("editor.htmlWrapColumn", 68);
             }
           }
           if (EnigmailCommon.getPref("wrapHtmlBeforeSend")) {
@@ -1829,7 +1864,7 @@ Enigmail.msg = {
           if (wrapWidth > 0 && wrapWidth < 68 && editor.wrapWidth > 0) {
             if (EnigmailCommon.confirmDlg(window, EnigmailCommon.getString("minimalLineWrapping", [ wrapWidth ] ))) {
               wrapWidth = 68;
-              EnigmailCommon.prefRoot.setIntPref("mailnews.wraplength", wrapWidth)
+              EnigmailCommon.prefRoot.setIntPref("mailnews.wraplength", wrapWidth);
             }
           }
 
@@ -1841,7 +1876,7 @@ Enigmail.msg = {
         }
         catch (ex) {}
       }
-    }https://addons.mozilla.org/en-US/developers/addon/enigmail/versions/1394284
+    }
     var exitCodeObj    = new Object();
     var statusFlagsObj = new Object();
     var errorMsgObj    = new Object();
@@ -1897,7 +1932,7 @@ Enigmail.msg = {
                      ? EnigmailCommon.convertFromUnicode(origText, charset)
                      : EnigmailCommon.convertFromUnicode(escText, charset);
 
-      var cipherText = enigmailSvc.encryptMessage(window, sendInfo.uiFlags, null, plainText,
+      var cipherText = enigmailSvc.encryptMessage(window, sendInfo.uiFlags, plainText,
                                             sendInfo.fromAddr, sendInfo.toAddr, sendInfo.bccAddr,
                                             sendInfo.sendFlags,
                                             exitCodeObj, statusFlagsObj,
@@ -2044,8 +2079,8 @@ Enigmail.msg = {
   modifyCompFields: function (msgCompFields)
   {
 
-  const HEADERMODE_KEYID = 0x01;
-  const HEADERMODE_URL   = 0x10;
+    const HEADERMODE_KEYID = 0x01;
+    const HEADERMODE_URL   = 0x10;
 
     try {
       if (this.identity.getBoolAttribute("enablePgp")) {
@@ -2085,16 +2120,28 @@ Enigmail.msg = {
   sendMessageListener: function (event)
   {
     EnigmailCommon.DEBUG_LOG("enigmailMsgComposeOverlay.js: Enigmail.msg.sendMessageListener\n");
-    try {
-      var msgcomposeWindow = document.getElementById("msgcomposeWindow");
-      this.modifyCompFields(gMsgCompose.compFields);
-      if (! this.encryptMsg(Number(msgcomposeWindow.getAttribute("msgtype")))) {
-        this.removeAttachedKey();
-        event.preventDefault();
-        event.stopPropagation();
+    let msgcomposeWindow = document.getElementById("msgcomposeWindow");
+    let sendMsgType = Number(msgcomposeWindow.getAttribute("msgtype"));
+
+    if (! (this.sendProcess && sendMsgType == Components.interfaces.nsIMsgCompDeliverMode.AutoSaveAsDraft)) {
+      this.sendProcess = true;
+
+      try {
+        this.modifyCompFields(gMsgCompose.compFields);
+        if (! this.encryptMsg(sendMsgType)) {
+          this.removeAttachedKey();
+          event.preventDefault();
+          event.stopPropagation();
+        }
       }
+      catch (ex) {}
     }
-    catch (ex) {}
+    else {
+      EnigmailCommon.DEBUG_LOG("enigmailMsgComposeOverlay.js: Enigmail.msg.sendMessageListener: sending in progress - autosave aborted\n");
+      event.preventDefault();
+      event.stopPropagation();
+    }
+    this.sendProcess = false;
   },
 
   // Replacement for wrong charset conversion detection of Thunderbird
@@ -2210,12 +2257,13 @@ Enigmail.msg = {
         return exitCodeObj.value;
       }
 
-      var fileInfo = new Object();
-      fileInfo.origFile  = origFile;
-      fileInfo.origUrl   = node.attachment.url;
-      fileInfo.origName  = node.attachment.name;
-      fileInfo.origTemp  = node.attachment.temporary;
-      fileInfo.origCType = node.attachment.contentType;
+      var fileInfo = {
+        origFile  : origFile,
+        origUrl   : node.attachment.url,
+        origName  : node.attachment.name,
+        origTemp  : node.attachment.temporary,
+        origCType : node.attachment.contentType
+      };
 
       // transform platform specific new file name to file:// URL
       var newUri = ioServ.newFileURI(newFile);
@@ -2275,7 +2323,7 @@ Enigmail.msg = {
     EnigmailCommon.DEBUG_LOG("enigmailMsgComposeOverlay.js: Enigmail.msg.toggleAccountAttr('"+attrName+"')\n");
 
     var oldValue = this.identity.getBoolAttribute(attrName);
-    this.identity.setBoolAttribute(attrName, !oldValue)
+    this.identity.setBoolAttribute(attrName, !oldValue);
 
   },
 
@@ -2451,7 +2499,7 @@ Enigmail.msg = {
                                                     nsIEnigmail.SIGNATURE_TEXT);
     }
 
-    var doubleDashSeparator = EnigmailCommon.getPref("doubleDashSeparator")
+    var doubleDashSeparator = EnigmailCommon.getPref("doubleDashSeparator");
     if (gMsgCompose.type != nsIMsgCompType.Template &&
         gMsgCompose.type != nsIMsgCompType.Draft &&
         doubleDashSeparator) {
@@ -2648,7 +2696,7 @@ Enigmail.msg = {
 
     Enigmail.msg.lastFocusedWindow = focusedWindow;
 
-    Enigmail.msg.fireSendFlags()
+    Enigmail.msg.fireSendFlags();
   },
 
   fireSendFlags: function ()
@@ -2658,7 +2706,7 @@ Enigmail.msg = {
       if (! this.determineSendFlagId) {
         this.determineSendFlagId = EnigmailCommon.dispatchEvent(
           function _sendFlagWrapper() {
-            Enigmail.msg.determineSendFlags()
+            Enigmail.msg.determineSendFlags();
           },
           0);
       }
@@ -2705,7 +2753,7 @@ Enigmail.composeStateListener = {
       NotifyDocumentStateChanged: function (nowDirty)
       {
       }
-    }
+    };
 
     var docStateListener = new enigDocStateListener();
 
@@ -2741,7 +2789,7 @@ Enigmail.composeStateListener = {
 
     if (!Enigmail.msg.timeoutId && !Enigmail.msg.dirty) {
       Enigmail.msg.timeoutId = EnigmailCommon.setTimeout(function () {
-          Enigmail.msg.decryptQuote(false)
+          Enigmail.msg.decryptQuote(false);
         },
         0);
     }
